@@ -273,6 +273,7 @@ struct struct_with_bitfields
   unsigned int g:1; // Compliant
   signed int   h:1; // 6.2 - signed int with size 1 is not compliant
   uint16_t     i:1; // Compliant
+  bool         j:1; // 6.1 - bool not legal until c99
 };
 
 static void misra6_1_fn(void) {
@@ -358,7 +359,7 @@ static int misra_8_2_g ( /* comment */ ); // 8.2
 static int misra_8_2_h ( /* comment 1 */ /* comment 2 */ ); // 8.2
 static int misra_8_2_i ( /* comment */ void);
 static int misra_8_2_j ( /* comment */ void /* comment */);
-static int misra_8_2_k ( // 
+static int misra_8_2_k ( //
  void);
 static int misra_8_2_l ( // 8.2
 );
@@ -395,6 +396,11 @@ uint8_t misra_8_4_buf2[24]; // 8.4
 typedef struct { uint16_t a; uint16_t b; } misra_8_4_struct;
 extern misra_8_4_struct bar[42];
 misra_8_4_struct bar[42]; // compliant
+extern uint16_t misra_8_4_speed = 6000u; //8.4
+uint8_t misra_8_4_pressure = 101u; //8.4
+int32_t misra_8_4_ext_val2;
+int32_t misra_8_4_ext_val2 = 3; // compliant
+int32_t misra_8_4_ext_val4; //8.4
 
 static int32_t misra_8_8 = 123;
 extern int32_t misra_8_8; // 8.8
@@ -441,6 +447,10 @@ static void misra_9_empty_or_zero_initializers(void) {
     int e[2][2] = { { 1 , 2 }, {} };           // 9.2
 
     int f[5]    = { 0 };
+    int f1[5]   = { 0u };                     // 9.3
+    unsigned int f1[ 3 ][ 2 ] = { 0U };       // 9.3 9.2
+    unsigned int f2[ 3 ] = { 0U };            // 9.3
+    float f3[ 3 ][ 2 ]  = { 0.0F };           // 9.3 9.2
     int g[5][2] = { 0 };
     int h[2][2] = { { 0 } };                   // 9.3
     int i[2][2] = { { 0 }, { 0 } };
@@ -497,6 +507,7 @@ static void misra_9_array_initializers_with_designators(void) {
     char    c[2][2]     = { [0] = {1, 2, 3} };
     char    d[1][2]     = { [0] = 1 };                                      // 9.2
     char    e[2][2]     = { { 1, 2 }, [1][0] = {3, 4} };                    // 9.2
+    int     e1[2][2]    = { [ 0 ][ 1 ] = 0, { 5, 6 } };                     // no warning #12419
     char    f[2]        = { [0] = 1, 2 };
     char    g[2]        = { [1] = 2, [0] = 1 };
     char    h[2][2]     = { { 1, 2 }, [1] = { 3 } };                        // 9.3
@@ -711,6 +722,7 @@ static void misra_10_3(uint32_t u32a, uint32_t u32b) {
     res = 2U + 3U; // no warning, utlr=unsigned char
     res = 0.1f; // 10.3
     const char c = '0'; // no-warning
+    bool b = true; // no-warning
     uint32_t u = UINT32_C(10); // no-warning
 }
 
@@ -725,7 +737,7 @@ static void misra_10_4(u32 x, s32 y) {
   z = x + y; //10.4
   z = (a == misra_10_4_A3) ? x : y; //10.4
   z = (a == misra_10_4_A3) ? y : y; // no-warning
-  
+
   // #10499
   const char buf[10] = {0};
   if ('0' == buf[x]) // no-warning
@@ -763,6 +775,7 @@ struct misra_10_6_s {
 static void misra_10_6(u8 x, char c1, char c2) {
   u16 y1 = x+x; // 10.6
   u16 y2 = (0x100u - 0x80u); // rhs is not a composite expression because it's a constant expression
+  int b = (y2 == y2) ? 0 : 1; // no-warning
   u16 z = ~u8 x ;//10.6
   s32 i = c1 - c2; // 10.3
   struct misra_10_6_s s;
@@ -830,10 +843,17 @@ static void misra_11_5(void *p) {
   p16 = p;     // 11.5
 }
 
+static intptr_t get_intptr_constant(void) { return 456; }
 static void misra_11_6(void) {
   void *p;
+  struct {
+    int i;
+  } s = { .i = 7 };
   p = (void*)123;  // 11.6
   x = (u64)p;      // 11.6
+  p = (void*)(1+1);// 11.6
+  p = (void*)get_intptr_constant(); // 11.6
+  p = (void*)s.i;  // 11.6
   p = ( void * )0; // no-warning
   (void)p;         // no-warning
   // # 12184
@@ -978,6 +998,9 @@ static void misra_12_4(uint8_t t) {
   x = MISRA12_4a + MISRA12_4b; // 12.4
   x = 0u - 1u; // 12.4
   x = t ? 0u : (0u-1u); // 12.4
+  x = (0u==0u) ? 0u : (0u-1u);
+  x = (0u!=0u) ? 0u : (0u-1u); // 12.4
+  x = (0u==0u) ? 0u : (2*(0u-1u)); // 10.4
 }
 
 struct misra_13_1_t { int a; int b; };
@@ -1964,6 +1987,24 @@ static void misra_21_16_f1(struct misra_21_16_S *s1, struct misra_21_16_S *s2) {
 static void misra_21_16_f2(char *x, char *y) {
     (void)memcmp(x, y, 10); // 21.16
 }
+typedef enum { R21_16_A, R21_16_B} r21_16_enum;
+static void misra_21_16_f3(void) {
+    int const a[2] = {0};
+    int const b[2] = {0};
+    (void)memcmp(a, b, 2); // no-warning
+    uint8_t const c[2] = {0};
+    uint8_t const d[2] = {0};
+    (void)memcmp(c, d, 2); // no-warning
+    bool const e[2] = {0};
+    bool const f[2] = {0};
+    (void)memcmp(e, f, 2); // no-warning
+    r21_16_enum const g[2] = {0};
+    r21_16_enum const h[2] = {0};
+    (void)memcmp(g, h, 2); // no-warning
+    char const i[2] = {0};
+    char const j[2] = {0};
+    (void)memcmp(i, j, 2); // 21.16
+}
 
 static void misra_21_19(void) {
     char *s = setlocale(LC_ALL,0); // 21.19
@@ -2022,3 +2063,13 @@ static void misra_22_10(void)
   f = strtod(inStr, NULL_PTR);
   if(errno != 0) {}
 }
+
+// #12448
+static void check_misra_config(void)
+{
+    if (sizeof(struct bar) == 0U) {} //no warning
+    if (sizeof(int abc) == 0U) {} //no warning
+    if (sizeof(xyz) == 0U) {} //no warning
+    if (sizeof(const pqr) == 0U) {} //no warning
+    if (sizeof(const int* const pqrs) == 0U) {} //no-warning
+}
