@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,15 +42,16 @@
 #include <algorithm>
 #include <cassert>
 #include <climits>
+#include <cstdint>
 #include <cstdio>
 #include <cstdlib> // EXIT_FAILURE
 #include <cstring>
-#include <fstream> // IWYU pragma: keep
+#include <fstream>
 #include <iostream>
 #include <iterator>
 #include <list>
 #include <set>
-#include <sstream> // IWYU pragma: keep
+#include <sstream>
 #include <unordered_set>
 #include <utility>
 
@@ -92,8 +93,8 @@ static bool addIncludePathsToList(const std::string& fileList, std::list<std::st
         // cppcheck-suppress accessMoved - FP
         while (std::getline(files, pathName)) { // next line
             if (!pathName.empty()) {
-                pathName = Path::removeQuotationMarks(pathName);
-                pathName = Path::fromNativeSeparators(pathName);
+                pathName = Path::removeQuotationMarks(std::move(pathName));
+                pathName = Path::fromNativeSeparators(std::move(pathName));
 
                 // If path doesn't end with / or \, add it
                 if (!endsWith(pathName, '/'))
@@ -134,11 +135,10 @@ namespace {
     };
 }
 
-CmdLineParser::CmdLineParser(CmdLineLogger &logger, Settings &settings, Suppressions &suppressions, Suppressions &suppressionsNoFail)
+CmdLineParser::CmdLineParser(CmdLineLogger &logger, Settings &settings, Suppressions &suppressions)
     : mLogger(logger)
     , mSettings(settings)
     , mSuppressions(suppressions)
-    , mSuppressionsNoFail(suppressionsNoFail)
 {}
 
 bool CmdLineParser::fillSettingsFromArgs(int argc, const char* const argv[])
@@ -184,7 +184,7 @@ bool CmdLineParser::fillSettingsFromArgs(int argc, const char* const argv[])
     // Output a warning for the user if he tries to exclude headers
     const std::vector<std::string>& ignored = getIgnoredPaths();
     const bool warn = std::any_of(ignored.cbegin(), ignored.cend(), [](const std::string& i) {
-        return Path::isHeader(i);
+        return Path::isHeader2(i);
     });
     if (warn) {
         mLogger.printMessage("filename exclusion does not apply to header (.h and .hpp) files.");
@@ -311,6 +311,9 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 {
     mSettings.exename = Path::getCurrentExecutablePath(argv[0]);
 
+    // default to --check-level=normal from CLI for now
+    mSettings.setCheckLevel(Settings::CheckLevel::normal);
+
     if (argc <= 1) {
         printHelp();
         return Result::Exit;
@@ -356,15 +359,8 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
         if (std::strcmp(argv[i], "--version") == 0) {
             if (!loadCppcheckCfg())
                 return Result::Fail;
-            if (!mSettings.cppcheckCfgProductName.empty()) {
-                mLogger.printRaw(mSettings.cppcheckCfgProductName);
-            } else {
-                const char * const extraVersion = CppCheck::extraVersion();
-                if (*extraVersion != '\0')
-                    mLogger.printRaw(std::string("Cppcheck ") + CppCheck::version() + " ("+ extraVersion + ')');
-                else
-                    mLogger.printRaw(std::string("Cppcheck ") + CppCheck::version());
-            }
+            const std::string version = getVersion();
+            mLogger.printRaw(version);
             return Result::Exit;
         }
     }
@@ -374,6 +370,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 
     ImportProject project;
 
+    bool executorAuto = true;
     int8_t logMissingInclude{0};
 
     for (int i = 1; i < argc; i++) {
@@ -432,8 +429,8 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                 else {
                     path = 2 + argv[i];
                 }
-                path = Path::removeQuotationMarks(path);
-                path = Path::fromNativeSeparators(path);
+                path = Path::removeQuotationMarks(std::move(path));
+                path = Path::fromNativeSeparators(std::move(path));
 
                 // If path doesn't end with / or \, add it
                 if (!endsWith(path,'/'))
@@ -474,19 +471,38 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
             else if (std::strcmp(argv[i], "--check-config") == 0)
                 mSettings.checkConfiguration = true;
 
-            // Check code exhaustively
-            else if (std::strcmp(argv[i], "--check-level=exhaustive") == 0)
-                mSettings.setCheckLevelExhaustive();
+            // Check level
+            else if (std::strncmp(argv[i], "--check-level=", 14) == 0) {
+                Settings::CheckLevel level = Settings::CheckLevel::normal;
+                const std::string level_s(argv[i] + 14);
+                if (level_s == "normal")
+                    level = Settings::CheckLevel::normal;
+                else if (level_s == "exhaustive")
+                    level = Settings::CheckLevel::exhaustive;
+                else {
+                    mLogger.printError("unknown '--check-level' value '" + level_s + "'.");
+                    return Result::Fail;
+                }
 
-            // Check code with normal analysis
-            else if (std::strcmp(argv[i], "--check-level=normal") == 0)
-                mSettings.setCheckLevelNormal();
+                mSettings.setCheckLevel(level);
+            }
 
             // Check library definitions
             else if (std::strcmp(argv[i], "--check-library") == 0) {
                 mSettings.checkLibrary = true;
             }
 
+            else if (std::strncmp(argv[i], "--check-version=", 16) == 0) {
+                if (!loadCppcheckCfg())
+                    return Result::Fail;
+                const std::string actualVersion = getVersion();
+                const std::string wantedVersion = argv[i] + 16;
+                if (actualVersion != wantedVersion) {
+                    mLogger.printError("--check-version check failed. Aborting.");
+                    return Result::Fail;
+                }
+            }
+
             else if (std::strncmp(argv[i], "--checkers-report=", 18) == 0)
                 mSettings.checkersReportFilename = argv[i] + 18;
 
@@ -614,6 +630,36 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 #endif
             }
 
+            else if (std::strncmp(argv[i], "--executor=", 11) == 0) {
+                const std::string type = 11 + argv[i];
+                if (type == "auto") {
+                    executorAuto = true;
+                    mSettings.executor = Settings::defaultExecutor();
+                }
+                else if (type == "thread") {
+#if defined(HAS_THREADING_MODEL_THREAD)
+                    executorAuto = false;
+                    mSettings.executor = Settings::ExecutorType::Thread;
+#else
+                    mLogger.printError("executor type 'thread' cannot be used as Cppcheck has not been built with a respective threading model.");
+                    return Result::Fail;
+#endif
+                }
+                else if (type == "process") {
+#if defined(HAS_THREADING_MODEL_FORK)
+                    executorAuto = false;
+                    mSettings.executor = Settings::ExecutorType::Process;
+#else
+                    mLogger.printError("executor type 'process' cannot be used as Cppcheck has not been built with a respective threading model.");
+                    return Result::Fail;
+#endif
+                }
+                else {
+                    mLogger.printError("unknown executor: '" + type + "'.");
+                    return Result::Fail;
+                }
+            }
+
             // Filter errors
             else if (std::strncmp(argv[i], "--exitcode-suppressions=", 24) == 0) {
                 // exitcode-suppressions=filename.txt
@@ -624,7 +670,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                     mLogger.printError("couldn't open the file: \"" + filename + "\".");
                     return Result::Fail;
                 }
-                const std::string errmsg(mSuppressionsNoFail.parseFile(f));
+                const std::string errmsg(mSuppressions.nofail.parseFile(f));
                 if (!errmsg.empty()) {
                     mLogger.printError(errmsg);
                     return Result::Fail;
@@ -632,8 +678,17 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
             }
 
             // use a file filter
-            else if (std::strncmp(argv[i], "--file-filter=", 14) == 0)
-                mSettings.fileFilters.emplace_back(argv[i] + 14);
+            else if (std::strncmp(argv[i], "--file-filter=", 14) == 0) {
+                const char *filter = argv[i] + 14;
+                if (std::strcmp(filter, "-") == 0) {
+                    if (!addFilesToList(filter, mSettings.fileFilters)) {
+                        mLogger.printError("Failed: --file-filter=-");
+                        return Result::Fail;
+                    }
+                } else {
+                    mSettings.fileFilters.emplace_back(filter);
+                }
+            }
 
             // file list specified
             else if (std::strncmp(argv[i], "--file-list=", 12) == 0) {
@@ -675,9 +730,9 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                 }
 
                 if (!path.empty()) {
-                    path = Path::removeQuotationMarks(path);
-                    path = Path::fromNativeSeparators(path);
-                    path = Path::simplifyPath(path);
+                    path = Path::removeQuotationMarks(std::move(path));
+                    path = Path::fromNativeSeparators(std::move(path));
+                    path = Path::simplifyPath(std::move(path));
 
                     if (Path::isDirectory(path)) {
                         // If directory name doesn't end with / or \, add it
@@ -751,7 +806,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
             }
 
             else if (std::strncmp(argv[i], "-l", 2) == 0) {
-#ifdef THREADING_MODEL_FORK
+#ifdef HAS_THREADING_MODEL_FORK
                 std::string numberString;
 
                 // "-l 3"
@@ -797,9 +852,9 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                 }
 
                 if (str == "c")
-                    mSettings.enforcedLang = Settings::Language::C;
+                    mSettings.enforcedLang = Standards::Language::C;
                 else if (str == "c++")
-                    mSettings.enforcedLang = Settings::Language::CPP;
+                    mSettings.enforcedLang = Standards::Language::CPP;
                 else {
                     mLogger.printError("unknown language '" + str + "' enforced.");
                     return Result::Fail;
@@ -946,11 +1001,6 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 
                     // keep existing platform from command-line intact
                     if (!platform.empty()) {
-                        if (platform == "Unspecified") {
-                            mLogger.printMessage("'Unspecified' is a deprecated platform type and will be removed in Cppcheck 2.14. Please use 'unspecified' instead.");
-                            platform = "unspecified";
-                        }
-
                         std::string errstr;
                         const std::vector<std::string> paths = {projectFile, argv[0]};
                         if (!mSettings.platform.set(platform, errstr, paths)) {
@@ -1035,6 +1085,12 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 #ifdef HAVE_RULES
                 Settings::Rule rule;
                 rule.pattern = 7 + argv[i];
+
+                if (rule.pattern.empty()) {
+                    mLogger.printError("no rule pattern provided.");
+                    return Result::Fail;
+                }
+
                 mSettings.rules.emplace_back(std::move(rule));
 #else
                 mLogger.printError("Option --rule cannot be used as Cppcheck has not been built with rules support.");
@@ -1045,42 +1101,77 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
             // Rule file
             else if (std::strncmp(argv[i], "--rule-file=", 12) == 0) {
 #ifdef HAVE_RULES
+                // TODO: improved error handling - wrong root node, etc.
+                // TODO: consume unused "version" attribute
                 const std::string ruleFile = argv[i] + 12;
                 tinyxml2::XMLDocument doc;
                 const tinyxml2::XMLError err = doc.LoadFile(ruleFile.c_str());
                 if (err == tinyxml2::XML_SUCCESS) {
-                    tinyxml2::XMLElement *node = doc.FirstChildElement();
+                    const tinyxml2::XMLElement *node = doc.FirstChildElement();
+                    // check if it is a single or multi rule configuration
                     if (node && strcmp(node->Value(), "rules") == 0)
                         node = node->FirstChildElement("rule");
                     for (; node && strcmp(node->Value(), "rule") == 0; node = node->NextSiblingElement()) {
                         Settings::Rule rule;
 
-                        const tinyxml2::XMLElement *tokenlist = node->FirstChildElement("tokenlist");
-                        if (tokenlist)
-                            rule.tokenlist = tokenlist->GetText();
-
-                        const tinyxml2::XMLElement *pattern = node->FirstChildElement("pattern");
-                        if (pattern) {
-                            rule.pattern = pattern->GetText();
+                        for (const tinyxml2::XMLElement *subnode = node->FirstChildElement(); subnode; subnode = subnode->NextSiblingElement()) {
+                            const char * const subtext = subnode->GetText();
+                            if (std::strcmp(subnode->Name(), "tokenlist") == 0) {
+                                rule.tokenlist = empty_if_null(subtext);
+                            }
+                            else if (std::strcmp(subnode->Name(), "pattern") == 0) {
+                                rule.pattern = empty_if_null(subtext);
+                            }
+                            else if (std::strcmp(subnode->Name(), "message") == 0) {
+                                for (const tinyxml2::XMLElement *msgnode = subnode->FirstChildElement(); msgnode; msgnode = msgnode->NextSiblingElement()) {
+                                    const char * const msgtext = msgnode->GetText();
+                                    if (std::strcmp(msgnode->Name(), "severity") == 0) {
+                                        rule.severity = severityFromString(empty_if_null(msgtext));
+                                    }
+                                    else if (std::strcmp(msgnode->Name(), "id") == 0) {
+                                        rule.id = empty_if_null(msgtext);
+                                    }
+                                    else if (std::strcmp(msgnode->Name(), "summary") == 0) {
+                                        rule.summary = empty_if_null(msgtext);
+                                    }
+                                    else {
+                                        mLogger.printError("unable to load rule-file '" + ruleFile + "' - unknown element '" + msgnode->Name() + "' encountered in 'message'.");
+                                        return Result::Fail;
+                                    }
+                                }
+                            }
+                            else {
+                                mLogger.printError("unable to load rule-file '" + ruleFile + "' - unknown element '" + subnode->Name() + "' encountered in 'rule'.");
+                                return Result::Fail;
+                            }
                         }
 
-                        tinyxml2::XMLElement *message = node->FirstChildElement("message");
-                        if (message) {
-                            const tinyxml2::XMLElement *severity = message->FirstChildElement("severity");
-                            if (severity)
-                                rule.severity = severityFromString(severity->GetText());
-
-                            const tinyxml2::XMLElement *id = message->FirstChildElement("id");
-                            if (id)
-                                rule.id = id->GetText();
-
-                            const tinyxml2::XMLElement *summary = message->FirstChildElement("summary");
-                            if (summary)
-                                rule.summary = summary->GetText() ? summary->GetText() : "";
+                        if (rule.pattern.empty()) {
+                            mLogger.printError("unable to load rule-file '" + ruleFile + "' - a rule is lacking a pattern.");
+                            return Result::Fail;
                         }
 
-                        if (!rule.pattern.empty())
-                            mSettings.rules.emplace_back(std::move(rule));
+                        if (rule.id.empty()) {
+                            mLogger.printError("unable to load rule-file '" + ruleFile + "' - a rule is lacking an id.");
+                            return Result::Fail;
+                        }
+
+                        if (rule.tokenlist.empty()) {
+                            mLogger.printError("unable to load rule-file '" + ruleFile + "' - a rule is lacking a tokenlist.");
+                            return Result::Fail;
+                        }
+
+                        if (rule.tokenlist != "normal" && rule.tokenlist != "define" && rule.tokenlist != "raw") {
+                            mLogger.printError("unable to load rule-file '" + ruleFile + "' - a rule is using the unsupported tokenlist '" + rule.tokenlist + "'.");
+                            return Result::Fail;
+                        }
+
+                        if (rule.severity == Severity::none) {
+                            mLogger.printError("unable to load rule-file '" + ruleFile + "' - a rule has an invalid severity.");
+                            return Result::Fail;
+                        }
+
+                        mSettings.rules.emplace_back(std::move(rule));
                     }
                 } else {
                     mLogger.printError("unable to load rule-file '" + ruleFile + "' (" + tinyxml2::XMLDocument::ErrorIDToName(err) + ").");
@@ -1143,7 +1234,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 
             else if (std::strncmp(argv[i], "--suppress=", 11) == 0) {
                 const std::string suppression = argv[i]+11;
-                const std::string errmsg(mSuppressions.addSuppressionLine(suppression));
+                const std::string errmsg(mSuppressions.nomsg.addSuppressionLine(suppression));
                 if (!errmsg.empty()) {
                     mLogger.printError(errmsg);
                     return Result::Fail;
@@ -1170,7 +1261,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                     mLogger.printError(message);
                     return Result::Fail;
                 }
-                const std::string errmsg(mSuppressions.parseFile(f));
+                const std::string errmsg(mSuppressions.nomsg.parseFile(f));
                 if (!errmsg.empty()) {
                     mLogger.printError(errmsg);
                     return Result::Fail;
@@ -1179,7 +1270,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 
             else if (std::strncmp(argv[i], "--suppress-xml=", 15) == 0) {
                 const char * filename = argv[i] + 15;
-                const std::string errmsg(mSuppressions.parseXmlFile(filename));
+                const std::string errmsg(mSuppressions.nomsg.parseXmlFile(filename));
                 if (!errmsg.empty()) {
                     mLogger.printError(errmsg);
                     return Result::Fail;
@@ -1208,6 +1299,9 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                     mSettings.templateFormat = "{file}:{line}:{column}: {severity}:{inconclusive:inconclusive:} {message} [{id}]\\n{code}";
                     mSettings.templateLocation = "{file}:{line}:{column}: note: {info}\\n{code}";
                     mSettings.daca = true;
+                } else if (mSettings.templateFormat == "simple") {
+                    mSettings.templateFormat = "{file}:{line}:{column}: {severity}:{inconclusive:inconclusive:} {message} [{id}]";
+                    mSettings.templateLocation = "";
                 }
                 // TODO: bail out when no placeholders are found?
             }
@@ -1276,6 +1370,10 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
     if (!loadCppcheckCfg())
         return Result::Fail;
 
+    // TODO: bail out?
+    if (!executorAuto && mSettings.useSingleJob())
+        mLogger.printMessage("'--executor' has no effect as only a single job will be used.");
+
     // Default template format..
     if (mSettings.templateFormat.empty()) {
         mSettings.templateFormat = "{bold}{file}:{line}:{column}: {red}{inconclusive:{magenta}}{severity}:{inconclusive: inconclusive:}{default} {message} [{id}]{reset}\\n{code}";
@@ -1540,6 +1638,7 @@ void CmdLineParser::printHelp() const
             "                          * misra-c-2012      Misra C 2012\n"
             "                          * misra-c-2023      Misra C 2023\n"
             "                          * misra-c++-2008    Misra C++ 2008\n"
+            "                          * misra-c++-2023    Misra C++ 2023\n"
             "                         Other:\n"
             "                          * bughunting        Soundy analysis\n"
             "                          * cert-c-int-precision=BITS  Integer precision to use in Cert C analysis.\n"
@@ -1688,9 +1787,18 @@ void CmdLineParser::printHelp() const
     mLogger.printRaw(oss.str());
 }
 
+std::string CmdLineParser::getVersion() const {
+    if (!mSettings.cppcheckCfgProductName.empty())
+        return mSettings.cppcheckCfgProductName;
+    const char * const extraVersion = CppCheck::extraVersion();
+    if (*extraVersion != '\0')
+        return std::string("Cppcheck ") + CppCheck::version() + " ("+ extraVersion + ')';
+    return std::string("Cppcheck ") + CppCheck::version();
+}
+
 bool CmdLineParser::isCppcheckPremium() const {
     if (mSettings.cppcheckCfgProductName.empty())
-        mSettings.loadCppcheckCfg();
+        Settings::loadCppcheckCfg(mSettings, mSettings.supprs);
     return startsWith(mSettings.cppcheckCfgProductName, "Cppcheck Premium");
 }
 
@@ -1729,6 +1837,9 @@ bool CmdLineParser::tryLoadLibrary(Library& destination, const std::string& base
         case Library::ErrorCode::PLATFORM_TYPE_REDEFINED:
             msg += "Platform type redefined";
             break;
+        case Library::ErrorCode::DUPLICATE_DEFINE:
+            msg += "Duplicate define";
+            break;
         }
         if (!err.reason.empty())
             msg += " '" + err.reason + "'";
@@ -1783,7 +1894,7 @@ bool CmdLineParser::loadAddons(Settings& settings)
 
 bool CmdLineParser::loadCppcheckCfg()
 {
-    const std::string cfgErr = mSettings.loadCppcheckCfg();
+    const std::string cfgErr = Settings::loadCppcheckCfg(mSettings, mSuppressions);
     if (!cfgErr.empty()) {
         mLogger.printError("could not load cppcheck.cfg - " + cfgErr);
         return false;
