@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 #include "applicationlist.h"
 #include "aboutdialog.h"
 #include "analyzerinfo.h"
+#include "checkstatistics.h"
+#include "checkthread.h"
 #include "common.h"
 #include "cppcheck.h"
 #include "errortypes.h"
@@ -38,6 +40,7 @@
 #include "report.h"
 #include "resultsview.h"
 #include "scratchpad.h"
+#include "settings.h"
 #include "showtypes.h"
 #include "statsdialog.h"
 #include "settingsdialog.h"
@@ -121,7 +124,7 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
     {
         Settings tempSettings;
         tempSettings.exename = QCoreApplication::applicationFilePath().toStdString();
-        tempSettings.loadCppcheckCfg(); // TODO: how to handle error?
+        Settings::loadCppcheckCfg(tempSettings, tempSettings.supprs); // TODO: how to handle error?
         mCppcheckCfgProductName = QString::fromStdString(tempSettings.cppcheckCfgProductName);
         mCppcheckCfgAbout = QString::fromStdString(tempSettings.cppcheckCfgAbout);
     }
@@ -247,7 +250,7 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
 
     for (int i = 0; i < mPlatforms.getCount(); i++) {
         PlatformData platform = mPlatforms.mPlatforms[i];
-        QAction *action = new QAction(this);
+        auto *action = new QAction(this);
         platform.mActMainWindow = action;
         mPlatforms.mPlatforms[i] = platform;
         action->setText(platform.mTitle);
@@ -392,10 +395,10 @@ void MainWindow::loadSettings()
     mUI->mActionToolBarFilter->setChecked(showFilterToolbar);
     mUI->mToolBarFilter->setVisible(showFilterToolbar);
 
-    const Settings::Language enforcedLanguage = (Settings::Language)mSettings->value(SETTINGS_ENFORCED_LANGUAGE, 0).toInt();
-    if (enforcedLanguage == Settings::CPP)
+    const Standards::Language enforcedLanguage = (Standards::Language)mSettings->value(SETTINGS_ENFORCED_LANGUAGE, 0).toInt();
+    if (enforcedLanguage == Standards::Language::CPP)
         mUI->mActionEnforceCpp->setChecked(true);
-    else if (enforcedLanguage == Settings::C)
+    else if (enforcedLanguage == Standards::Language::C)
         mUI->mActionEnforceC->setChecked(true);
     else
         mUI->mActionAutoDetectLanguage->setChecked(true);
@@ -469,11 +472,11 @@ void MainWindow::saveSettings() const
     mSettings->setValue(SETTINGS_TOOLBARS_FILTER_SHOW, mUI->mToolBarFilter->isVisible());
 
     if (mUI->mActionEnforceCpp->isChecked())
-        mSettings->setValue(SETTINGS_ENFORCED_LANGUAGE, Settings::CPP);
+        mSettings->setValue(SETTINGS_ENFORCED_LANGUAGE, Standards::Language::CPP);
     else if (mUI->mActionEnforceC->isChecked())
-        mSettings->setValue(SETTINGS_ENFORCED_LANGUAGE, Settings::C);
+        mSettings->setValue(SETTINGS_ENFORCED_LANGUAGE, Standards::Language::C);
     else
-        mSettings->setValue(SETTINGS_ENFORCED_LANGUAGE, Settings::None);
+        mSettings->setValue(SETTINGS_ENFORCED_LANGUAGE, Standards::Language::None);
 
     mApplications->saveSettings();
 
@@ -592,7 +595,7 @@ void MainWindow::doAnalyzeFiles(const QStringList &files, const bool checkLibrar
     mThread->setFiles(fileNames);
     if (mProjectFile && !checkConfiguration)
         mThread->setAddonsAndTools(mProjectFile->getAddonsAndTools());
-    mThread->setSuppressions(mProjectFile ? mProjectFile->getSuppressions() : QList<Suppressions::Suppression>());
+    mThread->setSuppressions(mProjectFile ? mProjectFile->getCheckingSuppressions() : QList<SuppressionList::Suppression>());
     QDir inf(mCurrentDirectory);
     const QString checkPath = inf.canonicalPath();
     setPath(SETTINGS_LAST_CHECK_PATH, checkPath);
@@ -821,24 +824,24 @@ void MainWindow::addIncludeDirs(const QStringList &includeDirs, Settings &result
     }
 }
 
-Library::Error MainWindow::loadLibrary(Library *library, const QString &filename)
+Library::Error MainWindow::loadLibrary(Library &library, const QString &filename)
 {
     Library::Error ret;
 
     // Try to load the library from the project folder..
     if (mProjectFile) {
         QString path = QFileInfo(mProjectFile->getFilename()).canonicalPath();
-        ret = library->load(nullptr, (path+"/"+filename).toLatin1());
+        ret = library.load(nullptr, (path+"/"+filename).toLatin1());
         if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
             return ret;
     }
 
     // Try to load the library from the application folder..
     const QString appPath = QFileInfo(QCoreApplication::applicationFilePath()).canonicalPath();
-    ret = library->load(nullptr, (appPath+"/"+filename).toLatin1());
+    ret = library.load(nullptr, (appPath+"/"+filename).toLatin1());
     if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
         return ret;
-    ret = library->load(nullptr, (appPath+"/cfg/"+filename).toLatin1());
+    ret = library.load(nullptr, (appPath+"/cfg/"+filename).toLatin1());
     if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
         return ret;
 
@@ -846,10 +849,10 @@ Library::Error MainWindow::loadLibrary(Library *library, const QString &filename
     // Try to load the library from FILESDIR/cfg..
     const QString filesdir = FILESDIR;
     if (!filesdir.isEmpty()) {
-        ret = library->load(nullptr, (filesdir+"/cfg/"+filename).toLatin1());
+        ret = library.load(nullptr, (filesdir+"/cfg/"+filename).toLatin1());
         if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
             return ret;
-        ret = library->load(nullptr, (filesdir+filename).toLatin1());
+        ret = library.load(nullptr, (filesdir+filename).toLatin1());
         if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
             return ret;
     }
@@ -858,10 +861,10 @@ Library::Error MainWindow::loadLibrary(Library *library, const QString &filename
     // Try to load the library from the cfg subfolder..
     const QString datadir = getDataDir();
     if (!datadir.isEmpty()) {
-        ret = library->load(nullptr, (datadir+"/"+filename).toLatin1());
+        ret = library.load(nullptr, (datadir+"/"+filename).toLatin1());
         if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
             return ret;
-        ret = library->load(nullptr, (datadir+"/cfg/"+filename).toLatin1());
+        ret = library.load(nullptr, (datadir+"/cfg/"+filename).toLatin1());
         if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
             return ret;
     }
@@ -869,7 +872,7 @@ Library::Error MainWindow::loadLibrary(Library *library, const QString &filename
     return ret;
 }
 
-bool MainWindow::tryLoadLibrary(Library *library, const QString& filename)
+bool MainWindow::tryLoadLibrary(Library &library, const QString& filename)
 {
     const Library::Error error = loadLibrary(library, filename);
     if (error.errorcode != Library::ErrorCode::OK) {
@@ -903,6 +906,9 @@ bool MainWindow::tryLoadLibrary(Library *library, const QString& filename)
         case Library::ErrorCode::PLATFORM_TYPE_REDEFINED:
             errmsg = tr("Platform type redefined");
             break;
+        case Library::ErrorCode::DUPLICATE_DEFINE:
+            errmsg = tr("Duplicate define");
+            break;
         case Library::ErrorCode::UNKNOWN_ELEMENT:
             errmsg = tr("Unknown element");
             break;
@@ -964,7 +970,10 @@ QPair<bool,Settings> MainWindow::getCppcheckSettings()
 
     result.exename = QCoreApplication::applicationFilePath().toStdString();
 
-    const bool std = tryLoadLibrary(&result.library, "std.cfg");
+    // default to --check-level=normal for GUI for now
+    result.setCheckLevel(Settings::CheckLevel::normal);
+
+    const bool std = tryLoadLibrary(result.library, "std.cfg");
     if (!std) {
         QMessageBox::critical(this, tr("Error"), tr("Failed to load %1. Your Cppcheck installation is broken. You can use --data-dir=<directory> at the command line to specify where this file is located. Please note that --data-dir is supposed to be used by installation scripts and therefore the GUI does not start when it is used, all that happens is that the setting is configured.\n\nAnalysis is aborted.").arg("std.cfg"));
         return {false, {}};
@@ -974,7 +983,7 @@ QPair<bool,Settings> MainWindow::getCppcheckSettings()
     const QString pythonCmd = fromNativePath(mSettings->value(SETTINGS_PYTHON_PATH).toString());
 
     {
-        const QString cfgErr = QString::fromStdString(result.loadCppcheckCfg());
+        const QString cfgErr = QString::fromStdString(Settings::loadCppcheckCfg(result, result.supprs));
         if (!cfgErr.isEmpty()) {
             QMessageBox::critical(this, tr("Error"), tr("Failed to load %1 - %2\n\nAnalysis is aborted.").arg("cppcheck.cfg").arg(cfgErr));
             return {false, {}};
@@ -1014,11 +1023,11 @@ QPair<bool,Settings> MainWindow::getCppcheckSettings()
         for (const QString& library : libraries) {
             result.libraries.emplace_back(library.toStdString());
             const QString filename = library + ".cfg";
-            tryLoadLibrary(&result.library, filename);
+            tryLoadLibrary(result.library, filename);
         }
 
-        for (const Suppressions::Suppression &suppression : mProjectFile->getSuppressions()) {
-            result.nomsg.addSuppression(suppression);
+        for (const SuppressionList::Suppression &suppression : mProjectFile->getCheckingSuppressions()) {
+            result.supprs.nomsg.addSuppression(suppression);
         }
 
         // Only check the given -D configuration
@@ -1045,7 +1054,7 @@ QPair<bool,Settings> MainWindow::getCppcheckSettings()
             result.platform.loadFromFile(applicationFilePath.toStdString().c_str(), platform.toStdString());
         } else {
             for (int i = Platform::Type::Native; i <= Platform::Type::Unix64; i++) {
-                const Platform::Type p = (Platform::Type)i;
+                const auto p = (Platform::Type)i;
                 if (platform == Platform::toString(p)) {
                     result.platform.set(p);
                     break;
@@ -1056,9 +1065,9 @@ QPair<bool,Settings> MainWindow::getCppcheckSettings()
         result.maxCtuDepth = mProjectFile->getMaxCtuDepth();
         result.maxTemplateRecursion = mProjectFile->getMaxTemplateRecursion();
         if (mProjectFile->isCheckLevelExhaustive())
-            result.setCheckLevelExhaustive();
+            result.setCheckLevel(Settings::CheckLevel::exhaustive);
         else
-            result.setCheckLevelNormal();
+            result.setCheckLevel(Settings::CheckLevel::normal);
         result.checkHeaders = mProjectFile->getCheckHeaders();
         result.checkUnusedTemplates = mProjectFile->getCheckUnusedTemplates();
         result.safeChecks.classes = mProjectFile->safeChecks.classes;
@@ -1087,6 +1096,7 @@ QPair<bool,Settings> MainWindow::getCppcheckSettings()
             if (!premiumArgs.contains("misra") && mProjectFile->getAddons().contains("misra"))
                 premiumArgs += " --misra-c-2012";
             result.premiumArgs = premiumArgs.mid(1).toStdString();
+            result.setMisraRuleTexts(CheckThread::executeCommand);
         }
     }
 
@@ -1119,7 +1129,7 @@ QPair<bool,Settings> MainWindow::getCppcheckSettings()
         result.platform.set((Platform::Type) mSettings->value(SETTINGS_CHECKED_PLATFORM, 0).toInt());
     result.standards.setCPP(mSettings->value(SETTINGS_STD_CPP, QString()).toString().toStdString());
     result.standards.setC(mSettings->value(SETTINGS_STD_C, QString()).toString().toStdString());
-    result.enforcedLang = (Settings::Language)mSettings->value(SETTINGS_ENFORCED_LANGUAGE, 0).toInt();
+    result.enforcedLang = (Standards::Language)mSettings->value(SETTINGS_ENFORCED_LANGUAGE, 0).toInt();
 
     if (result.jobs <= 1) {
         result.jobs = 1;
@@ -1163,7 +1173,7 @@ void MainWindow::analysisDone()
         if (QDir(buildDir).exists()) {
             mUI->mResults->saveStatistics(buildDir + "/statistics.txt");
             mUI->mResults->updateFromOldReport(buildDir + "/lastResults.xml");
-            mUI->mResults->save(buildDir + "/lastResults.xml", Report::XMLV2);
+            mUI->mResults->save(buildDir + "/lastResults.xml", Report::XMLV2, mCppcheckCfgProductName);
         }
     }
 
@@ -1277,7 +1287,7 @@ void MainWindow::reAnalyzeSelected(const QStringList& files)
     QDateTime saveCheckStartTime = mThread->getCheckStartTime();
     mThread->check(checkSettings);
     mUI->mResults->setCheckSettings(checkSettings);
-    mThread->setCheckStartTime(saveCheckStartTime);
+    mThread->setCheckStartTime(std::move(saveCheckStartTime));
 }
 
 void MainWindow::reAnalyze(bool all)
@@ -1504,21 +1514,21 @@ void MainWindow::about()
         msg.exec();
     }
     else {
-        AboutDialog *dlg = new AboutDialog(CppCheck::version(), CppCheck::extraVersion(), this);
+        auto *dlg = new AboutDialog(CppCheck::version(), CppCheck::extraVersion(), this);
         dlg->exec();
     }
 }
 
 void MainWindow::showLicense()
 {
-    FileViewDialog *dlg = new FileViewDialog(":COPYING", tr("License"), this);
+    auto *dlg = new FileViewDialog(":COPYING", tr("License"), this);
     dlg->resize(570, 400);
     dlg->exec();
 }
 
 void MainWindow::showAuthors()
 {
-    FileViewDialog *dlg = new FileViewDialog(":AUTHORS", tr("Authors"), this);
+    auto *dlg = new FileViewDialog(":AUTHORS", tr("Authors"), this);
     dlg->resize(350, 400);
     dlg->exec();
 }
@@ -1561,7 +1571,7 @@ void MainWindow::save()
                 type = Report::CSV;
         }
 
-        mUI->mResults->save(selectedFile, type);
+        mUI->mResults->save(selectedFile, type, mCppcheckCfgProductName);
         setPath(SETTINGS_LAST_RESULT_PATH, selectedFile);
     }
 }
@@ -1582,9 +1592,9 @@ void MainWindow::complianceReport()
     tempResults.open();
     tempResults.close();
 
-    mUI->mResults->save(tempResults.fileName(), Report::XMLV2);
+    mUI->mResults->save(tempResults.fileName(), Report::XMLV2, mCppcheckCfgProductName);
 
-    ComplianceReportDialog dlg(mProjectFile, tempResults.fileName());
+    ComplianceReportDialog dlg(mProjectFile, tempResults.fileName(), mUI->mResults->getStatistics()->getCheckersReport());
     dlg.exec();
 }
 
@@ -1670,7 +1680,7 @@ void MainWindow::openHelpContents()
 
 void MainWindow::openOnlineHelp()
 {
-    HelpDialog *helpDialog = new HelpDialog;
+    auto *helpDialog = new HelpDialog;
     helpDialog->showMaximized();
 }
 
@@ -1745,7 +1755,7 @@ void MainWindow::analyzeProject(const ProjectFile *projectFile, const bool check
     Settings::terminate(false);
 
     QFileInfo inf(projectFile->getFilename());
-    const QString rootpath = projectFile->getRootPath();
+    const QString& rootpath = projectFile->getRootPath();
 
     QDir::setCurrent(inf.absolutePath());
 
@@ -1959,7 +1969,7 @@ void MainWindow::enableProjectOpenActions(bool enable)
 
 void MainWindow::openRecentProject()
 {
-    QAction *action = qobject_cast<QAction *>(sender());
+    auto *action = qobject_cast<QAction *>(sender());
     if (!action)
         return;
     const QString project = action->data().toString();
@@ -2048,7 +2058,7 @@ void MainWindow::removeProjectMRU(const QString &project)
 
 void MainWindow::selectPlatform()
 {
-    QAction *action = qobject_cast<QAction *>(sender());
+    auto *action = qobject_cast<QAction *>(sender());
     if (action) {
         const Platform::Type platform = (Platform::Type) action->data().toInt();
         mSettings->setValue(SETTINGS_CHECKED_PLATFORM, platform);
@@ -2061,7 +2071,7 @@ void MainWindow::suppressIds(QStringList ids)
         return;
     ids.removeDuplicates();
 
-    QList<Suppressions::Suppression> suppressions = mProjectFile->getSuppressions();
+    QList<SuppressionList::Suppression> suppressions = mProjectFile->getSuppressions();
     for (const QString& id : ids) {
         // Remove all matching suppressions
         std::string id2 = id.toStdString();
@@ -2072,7 +2082,7 @@ void MainWindow::suppressIds(QStringList ids)
                 ++i;
         }
 
-        Suppressions::Suppression newSuppression;
+        SuppressionList::Suppression newSuppression;
         newSuppression.errorId = id2;
         suppressions << newSuppression;
     }
