@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,19 +29,23 @@
 #include "settings.h"
 
 #include <cstddef>
-#include <fstream> // IWYU pragma: keep
+#include <fstream>
 #include <functional>
 #include <list>
 #include <map>
+#include <memory>
 #include <set>
 #include <string>
 #include <unordered_set>
-#include <utility>
 #include <vector>
 
-class Tokenizer;
+class TokenList;
 enum class SHOWTIME_MODES;
 struct FileSettings;
+class CheckUnusedFunctions;
+class Tokenizer;
+
+namespace simplecpp { class TokenList; }
 
 /// @addtogroup Core
 /// @{
@@ -138,11 +142,7 @@ public:
     void analyseClangTidy(const FileSettings &fileSettings);
 
     /** analyse whole program use .analyzeinfo files */
-    void analyseWholeProgram(const std::string &buildDir, const std::list<std::pair<std::string, std::size_t>> &files, const std::list<FileSettings>& fileSettings);
-
-    /** Check if the user wants to check for unused functions
-     * and if it's possible at all */
-    bool isUnusedFunctionCheckEnabled() const;
+    unsigned int analyseWholeProgram(const std::string &buildDir, const std::list<std::pair<std::string, std::size_t>> &files, const std::list<FileSettings>& fileSettings);
 
     /** Remove *.ctu-info files */
     void removeCtuInfoFiles(const std::list<std::pair<std::string, std::size_t>>& files, const std::list<FileSettings>& fileSettings); // cppcheck-suppress functionConst // has side effects
@@ -154,6 +154,11 @@ public:
 
     std::string getAddonMessage(const std::string& id, const std::string& text) const;
 
+    /**
+     * @brief Get dumpfile <rawtokens> contents, this is only public for testing purposes
+     */
+    std::string getDumpFileContentsRawTokens(const std::vector<std::string>& files, const simplecpp::TokenList& tokens1) const;
+
 private:
 #ifdef HAVE_RULES
     /** Are there "simple" rules */
@@ -172,12 +177,6 @@ private:
      */
     unsigned int checkFile(const std::string& filename, const std::string &cfgname, std::istream* fileStream = nullptr);
 
-    /**
-     * @brief Check raw tokens
-     * @param tokenizer tokenizer instance
-     */
-    void checkRawTokens(const Tokenizer &tokenizer);
-
     /**
      * @brief Check normal tokens
      * @param tokenizer tokenizer instance
@@ -198,12 +197,14 @@ private:
 #ifdef HAVE_RULES
     /**
      * @brief Execute rules, if any
-     * @param tokenlist token list to use (normal / simple)
-     * @param tokenizer tokenizer
+     * @param tokenlist token list to use (define / normal / raw)
+     * @param list token list
      */
-    void executeRules(const std::string &tokenlist, const Tokenizer &tokenizer);
+    void executeRules(const std::string &tokenlist, const TokenList &list);
 #endif
 
+    unsigned int checkClang(const std::string &path);
+
     /**
      * @brief Errors and warnings are directed here.
      *
@@ -250,6 +251,8 @@ private:
     ExecuteCmdFn mExecuteCommand;
 
     std::ofstream mPlistFile;
+
+    std::unique_ptr<CheckUnusedFunctions> mUnusedFunctionsCheck;
 };
 
 /// @}
